@@ -490,6 +490,20 @@ void ObjectPad_mop_class_compose_role(pTHX_ ClassMeta *classmeta, ClassMeta *rol
   }
 }
 
+#define embed_slothook(roleh, offset)  S_embed_slothook(aTHX_ roleh, offset)
+static struct SlotHook *S_embed_slothook(pTHX_ struct SlotHook *roleh, SLOTOFFSET offset)
+{
+  struct SlotHook *classh;
+  Newx(classh, 1, struct SlotHook);
+
+  classh->slotix   = roleh->slotix + offset;
+  classh->slotmeta = roleh->slotmeta;
+  classh->funcs    = roleh->funcs;
+  classh->hookdata = roleh->hookdata;
+
+  return classh;
+}
+
 #define mop_class_apply_role(embedding)  S_mop_class_apply_role(aTHX_ embedding)
 static void S_mop_class_apply_role(pTHX_ RoleEmbedding *embedding)
 {
@@ -537,6 +551,28 @@ static void S_mop_class_apply_role(pTHX_ RoleEmbedding *embedding)
     }
   }
 
+  if(rolemeta->slothooks_postslots) {
+    if(!classmeta->slothooks_postslots)
+      classmeta->slothooks_postslots = newAV();
+
+    U32 i;
+    for(i = 0; i < av_count(rolemeta->slothooks_postslots); i++) {
+      struct SlotHook *roleh = (struct SlotHook *)AvARRAY(rolemeta->slothooks_postslots)[i];
+      av_push(classmeta->slothooks_postslots, (SV *)embed_slothook(roleh, embedding->offset));
+    }
+  }
+
+  if(rolemeta->slothooks_construct) {
+    if(!classmeta->slothooks_construct)
+      classmeta->slothooks_construct = newAV();
+
+    U32 i;
+    for(i = 0; i < av_count(rolemeta->slothooks_construct); i++) {
+      struct SlotHook *roleh = (struct SlotHook *)AvARRAY(rolemeta->slothooks_construct)[i];
+      av_push(classmeta->slothooks_construct, (SV *)embed_slothook(roleh, embedding->offset));
+    }
+  }
+
   classmeta->next_slotix += av_count(rolemeta->slots);
 
   /* TODO: Run an APPLY block if the role has one */
@@ -865,6 +901,7 @@ void ObjectPad_mop_class_seal(pTHX_ ClassMeta *meta)
           Newx(fasth, 1, struct SlotHook);
 
           fasth->slotix   = slotix;
+          fasth->slotmeta = slotmeta;
           fasth->funcs    = h->funcs;
           fasth->hookdata = h->hookdata;
 
@@ -879,6 +916,7 @@ void ObjectPad_mop_class_seal(pTHX_ ClassMeta *meta)
           Newx(fasth, 1, struct SlotHook);
 
           fasth->slotix   = slotix;
+          fasth->slotmeta = slotmeta;
           fasth->funcs    = h->funcs;
           fasth->hookdata = h->hookdata;
 
@@ -916,6 +954,8 @@ XS_INTERNAL(injected_constructor)
   SV *self = NULL;
 
   assert(meta->type == METATYPE_CLASS);
+  if(!meta->sealed)
+    croak("Cannot yet invoke '%" SVf "' constructor before the class is complete", SVfARG(class));
 
   COP *prevcop = PL_curcop;
   PL_curcop = meta->tmpcop;
@@ -1088,15 +1128,13 @@ XS_INTERNAL(injected_constructor)
     CopLINE_set(PL_curcop, __LINE__);
 
     AV *slothooks = meta->slothooks_postslots;
-    AV *slots     = meta->slots;
 
     U32 i;
     for(i = 0; i < av_count(slothooks); i++) {
       struct SlotHook *h = (struct SlotHook *)AvARRAY(slothooks)[i];
       SLOTOFFSET slotix = h->slotix;
-      struct SlotMeta *slotmeta = (struct SlotMeta *)AvARRAY(slots)[slotix];
 
-      (*h->funcs->post_initslot)(aTHX_ slotmeta, h->hookdata, slotsv[slotix]);
+      (*h->funcs->post_initslot)(aTHX_ h->slotmeta, h->hookdata, slotsv[slotix]);
     }
   }
 
@@ -1222,15 +1260,13 @@ XS_INTERNAL(injected_constructor)
     CopLINE_set(PL_curcop, __LINE__);
 
     AV *slothooks = meta->slothooks_construct;
-    AV *slots     = meta->slots;
 
     U32 i;
     for(i = 0; i < av_count(slothooks); i++) {
       struct SlotHook *h = (struct SlotHook *)AvARRAY(slothooks)[i];
       SLOTOFFSET slotix = h->slotix;
-      struct SlotMeta *slotmeta = (struct SlotMeta *)AvARRAY(slots)[slotix];
 
-      (*h->funcs->post_construct)(aTHX_ slotmeta, h->hookdata, slotsv[slotix]);
+      (*h->funcs->post_construct)(aTHX_ h->slotmeta, h->hookdata, slotsv[slotix]);
     }
   }
 
@@ -1439,25 +1475,31 @@ ClassMeta *ObjectPad_mop_create_class(pTHX_ enum MetaType type, SV *name, SV *su
       meta->cls.foreign_new = supermeta->cls.foreign_new;
 
       if(supermeta->buildblocks) {
-        AV *superbuildblocks = supermeta->buildblocks;
-        U32 i;
-
         if(!meta->buildblocks)
           meta->buildblocks = newAV();
 
-        for(i = 0; i < av_count(superbuildblocks); i++)
-          av_push(meta->buildblocks, /* no inc */ AvARRAY(superbuildblocks)[i]);
+        av_push_from_av_noinc(meta->buildblocks, supermeta->buildblocks);
       }
 
       if(supermeta->adjustblocks) {
-        AV *superadjustblocks = supermeta->adjustblocks;
-        U32 i;
-
         if(!meta->adjustblocks)
           meta->adjustblocks = newAV();
 
-        for(i = 0; i < av_count(superadjustblocks); i++)
-          av_push(meta->adjustblocks, /* no inc */ AvARRAY(superadjustblocks)[i]);
+        av_push_from_av_noinc(meta->adjustblocks, supermeta->adjustblocks);
+      }
+
+      if(supermeta->slothooks_postslots) {
+        if(!meta->slothooks_postslots)
+          meta->slothooks_postslots = newAV();
+
+        av_push_from_av_noinc(meta->slothooks_postslots, supermeta->slothooks_postslots);
+      }
+
+      if(supermeta->slothooks_construct) {
+        if(!meta->slothooks_construct)
+          meta->slothooks_construct = newAV();
+
+        av_push_from_av_noinc(meta->slothooks_construct, supermeta->slothooks_construct);
       }
 
       if(supermeta->parammap) {
